module libGraphics =
   pragmas 
      COMMENT "Predefined library for simulating graphically the execution of Signal program."
     "The graphical part is written in Java to make the simulation program portable."

   end pragmas
   type TWindow = integer;
   type Direction = enum (horiz, vertic);
   use Colors; 
   process createFrame =
        ( ? integer width, height;
          ! TWindow win;
        )
      spec(| width ^= height ^= win |)
      pragmas 
         COMMENT "It creates a frame, where all the graphics will be built(buttons, canvas)."
                 "width: width of the frame; height: heigth of the frame."
                 "win: the window of the created object."
                 "The call to createFrame is obligatory; if you use the pacemaker, this call is already done"
         Java_Code "&o1 = sim.&n(&i1, &i2)"
         Java_Code_Import "import SignalLibraries.Java.graphical.*"
         Java_Code_Declare "Simu sim = new Simu()"
      end pragmas
   
   ; 
   process createPanel =
        ( ? TWindow win;
            Direction d;
          ! TWindow panel;
        )
      spec(| win ^= d ^= panel |)
      pragmas 
         COMMENT "It creates a panel, that means a window where others graphical components can be put."
                 "win: mother window;  d: the direction (horizontal and vertical)"
                 "panel: the window of the created object."
         Java_Code "&o1 = sim.&n(&i1,&i2)"
      end pragmas
   
   ; 
   process createPanelWithCustomLayout =
        ( ? TWindow win;
            integer d;
          ! TWindow panel;
        )
      spec(| win ^= d ^= panel |)
      pragmas 
         COMMENT "It creates a panel, that means a window where others graphical components can be put."
                 "win: mother window;  d: the direction (horizontal and vertical)"
                 "panel: the window of the created object."
         Java_Code "&o1 = sim.&n(&i1,&i2)"
      end pragmas
   
   ; 
   process createBorder =
        ( ? TWindow win;
            integer width, height;
          ! integer wout;
        )
      spec(| win ^= width ^= height ^= wout |)
      pragmas 
         COMMENT "It creates a blank area (allows to align and separate some objects)"
                 "win mother window, width: width of the area, height:  height of the area."
                 "wout: the window of the created object."
         Java_Code "sim.&n(&i1,&i2,&i3)"
      end pragmas
   
   ; 
   process createButton =
        ( ? TWindow win;
          ! TWindow but;
        )
      spec(| win ^= but |)
      pragmas 
         COMMENT "It creates a button."
                 "win: mother window."
                 "but: the window of the created object."
         Java_Code "&o1=sim.&n(&i1)"
      end pragmas
   
      ; 
       process createPlan =
         ( ? TWindow win;
             string imageFileName;
          ! TWindow but;
        )
      spec(| win ^= but ^= imageFileName |)
      pragmas 
         COMMENT "It creates a Plan with the image of the file imageFileName."
                 "win: mother window."
                 "but: the window of the created object."
         Java_Code "&o1=sim.&n(&i1, &i2)"
      end pragmas
   
   ; 

   process createCheckBox =
        ( ? TWindow win;
            boolean b;
          ! TWindow cB;
        )
      spec(| win ^= b ^= cB |)
      pragmas 
         COMMENT "It creates a button with a box to check."
                 "win: mother window, b: initial state (check or uncheck)"
                 "cB: the window of the created object."
         Java_Code "&o1=sim.&n(&i1,&i2)"
      end pragmas
   
   ; 
   process createSlider =
        ( ? TWindow win;
            integer min, max, vinit, width, height, tickDisplay;
          ! TWindow vlabel;
        )
      spec(| win ^= min ^= max ^= vinit ^= width ^= height ^= tickDisplay ^= vlabel |)
      pragmas 
         COMMENT "It creates a slider (or cursor)"
                 "win: mother window, min:the minimum value, max:the maximum value, vinit: initial value,"
                 "width: the width,heigth: the height, tickDisplay:the distance between 2 ticks drawn"
                 "vlabel: the window of the created object."
         Java_Code "&o1=sim.&n(&i1,&i2,&i3,&i4,&i5,&i6, &i7)"
      end pragmas
   
   ; 
   process createComboBox =
        ( ? TWindow win;
            integer MIN_A, MAX_A, pas, height, tickDisplay;
          ! TWindow vlabel;
        )
      spec(| win ^= MIN_A ^= MAX_A ^= pas ^= height ^= tickDisplay ^= vlabel |)
      pragmas 
         Java_Code "&o1=sim.&n(&i1,&i2,&i3,&i4,&i5,&i6)"
      end pragmas
   
   ; 
   process createLabel =
        ( ? TWindow win;
          ! TWindow vlabel;
        )
      spec(| win ^= vlabel |)
      pragmas 
         COMMENT "It creates a label, that means a text area that cannot be modified."
                 "win: mother window"
                 "vlabel: the window of the created object."
         Java_Code "&o1=sim.&n(&i1)"
      end pragmas
   
   ; 
   process createLabelstr =
        ( ? TWindow win;
            string s;
          ! TWindow vlabel;
        )
      spec (| win ^= s ^= vlabel |)
      pragmas 
         Java_Code "&o1=sim.&n(&i1, &i2)"
      end pragmas
   
   ; 
   process createCanvas =
        ( ? TWindow win;
            integer width, height;
          ! TWindow canvas;
        )
      spec(| win ^= width ^= height ^= canvas |)
      pragmas 
         COMMENT "It creates a canvas (to draw curves or patterns)"
                 "win: mother window, width: the width of the canvas, height: height of the canvas"
                 "canvas: the window of the created object."
         Java_Code "&o1=sim.&n(&i1,&i2,&i3)"
      end pragmas
   
   ; 
   process createTextArea =
        ( ? TWindow win;
            integer width, height;
          ! TWindow text;
        )
      spec(| win ^= width ^= height ^= text |)
      pragmas 
         COMMENT "It creates a text area that can be modified."
                 "win: mother window,  width: the width of the area, height: height of the area."
                 "text:the window of the created object."
         Java_Code "&o1=sim.&n(&i1,&i2,&i3)"
      end pragmas
   
   ; 
   process readButton =
        ( ? TWindow w;
          ! boolean b;
        )
      spec(| w ^= b |)
      pragmas 
         COMMENT "Getting of the state of a button."
                 "w: window assigned to the button."
                 "b: true if the button has been clicked, false otherwise."
         Java_Code "&o1=sim.&n(&i1)"
      end pragmas
   
   ; 
   process readCheckBox =
        ( ? TWindow w;
          ! boolean b;
        )
      spec(| w ^= b |)
      pragmas 
         COMMENT "Getting of the state of a checkbox."
                 "w: window assigned to the button to be checked."
                 "b: true if the button is checked, false otherwise."
         Java_Code "&o1=sim.&n(&i1)"
      end pragmas
   
   ; 
   function readSlider =
        ( ? TWindow w;
          ! integer v;
        )
      spec(| w ^= v |)
      pragmas 
         COMMENT "Reading of the value of a slider."
                 "w: window of the cursor."
                 "v: the value of the cursor."
         Java_Code "&o1=sim.&n(&i1)"
      end pragmas
   
   ; 
   function readComboBox =
        ( ? TWindow w;
          ! dreal v;
        )
      spec(| w ^= v |)
      pragmas 
         Java_Code "&o1=sim.&n(&i1)"
      end pragmas
   
   ; 
   process setTextArea =
        ( ? TWindow w;
            integer val;
        )
        spec (| w ^= val |)
      pragmas 
         COMMENT "Writing an integer"
                 "w:window of the text area, val: integer value to be writen."
         Java_Code "sim.&n(&i1,&i2)"
      end pragmas
   
   ; 
   process drawLine =
        ( ? TWindow w;
            integer x1, y1, x2, y2;
            TColor vcolor;
        )
        spec (| w ^= x1 ^= y1 ^= y2 ^= x2 ^= vcolor |)
      pragmas 
         COMMENT "Drawing a line."
                 "w:window of the canvas."
                 "x1, y1: the X-coordinate (resp. Y-coordinate) of the start point."
                 "x2, y2: the X-coordinate (resp. Y-coordinate) of the arrival point."
         Java_Code "sim.drawLine(&i1,&i2,&i3,&i4,&i5, &i6)"
      end pragmas
   
   ; 
   process drawVertLine =
        ( ? TWindow w;
            integer x1, y1, y2;
            TColor vcolor;
        )
        spec (| w ^= x1 ^= y1 ^= y2 |)
      pragmas 
         COMMENT "Drawing a vertical line in a canvas."
                 "w:window of the canvas, x1:the X-coordinate of the line"
                 "y1: the Y-coordinate of the start point."
                 "y2: the Y-coordinate of the arrival point."
      end pragmas
      (| drawLine(w,x1,y1,x1,y2,vcolor) |)
   ; 
   process drawHorLine =
        ( ? TWindow w;
            integer x1, y1, x2;
            TColor vcolor;
        )
        spec (| w ^= x1 ^= y1 ^= x2 |)
      pragmas 
         COMMENT "Drawing a horizontal line in a canvas."
                 "w:window of the canvas, x1: the X-coordinate of the start point,"
                 "y1: the Y-coordinate of the start point,"
                 "x2: the X-coordinate of the arrival point."
      end pragmas
      (| drawLine(w,x1,y1,x2,y1,vcolor) |)
   ; 
   process clearCanvas =
        ( ? TWindow w;
            integer x1, dx, dy;
        )
        spec (| w ^= x1 ^= dy ^= dx |)
      pragmas 
         COMMENT "Resetting a rectangular area in a canvas"
                 "w: window of the canvas, x1:the X-coordinate of the start point,"
                 "dx: the X-coordinate of the arrival point, dy: the Y-coordinate"
         Java_Code "sim.&n(&i1,&i2,&i3,&i4)"
      end pragmas
   
   ; 
   process fillRect =
        ( ? TWindow w;
            integer x, y;
            integer dx, dy;
            TColor vcolor;
        )
        spec (| w ^= x ^= y ^= dx ^= dy |)
      pragmas 
         COMMENT "Writing a full rectangle in black color in a canvas"
                 "w: window of the canvas,"
                 "x, y:the X (resp. Y-coordinate) of the start point,"
                 "dx,dy: the X (resp. Y-coordinate) of the arrival point.??? length???"
         Java_Code "sim.&n(&i1,&i2,&i3,&i4,&i5, &i6)"
      end pragmas
   
   ; 
   process drawInteger =
        ( ? TWindow w;
            integer val;
            integer x, y;
        )
        spec (| w ^= val ^= y ^= x |)
      pragmas 
         COMMENT "Writing an integer in a canvas"
                 "w: window of the canvas, val: the value to be written,"
                 "x: position in X-coordinate,y: position in Y-coordinate"
         Java_Code "sim.drawInteger(&i1,&i2,&i3,&i4)"
      end pragmas
   
   ; 
   process drawReal =
        ( ? TWindow w;
            dreal ch;
            integer x, y;
        )
        spec (| w ^= ch ^= y ^= x |)
      pragmas 
         COMMENT "Writing an real in a canvas"
                 "w: window of the canvas, val: the value to be written,"
                 "x: position in X-coordinate,y: position in Y-coordinate"
         Java_Code "sim.drawReal(&i1,&i2,&i3,&i4)"
      end pragmas
   
   ; 
   process quitAppli =
        ( ? event h;
        )
      pragmas 
         COMMENT "Exiting function:it terminates the simulation."
                 "h: the activation clock."
         Java_Code "sim.&n()"
      end pragmas
   
   ; 
   process PACE_MAKER =
        ( ? event HE;
          ! TWindow frame;
            event HES;
            event H_0;
        )
      pragmas 
         COMMENT "Definition of a graphical pace-maker with the following buttons:"
                 "  - AUTO/MANU  : switches the mode (AUTOMATIC/MANUAL)"
                 "  - TICK       : gives the ticks in MANUAL mode"
                 "                 and gives supplementary ticks in AUTOMATIC mode"
                 "  - QUIT       : quits the program in MANUAL mode"
                 "  - +          : decreases the period of the clock program"
                 "                 (basic frequency in AUTOMATIC mode)"
                 "  - -          : increases the period of the clock program"
                 "  - /          : divides by 2 the period of the clock program"
                 "  - *          : doubles the period of the clock program"
                 " (buttons +, -, /, * can be depressed in both AUTOMATIC and MANUAL modes)"
                 " HE : Master Clock of the simulation."
                 " frame: the window of the created object."
                 " HES : sub-clock of HE obtained through the command buttons."
                 " H_0 : clock of the initial instant."
      end pragmas
      (| (| (| HE := HS
             | PACE_MAKER{}
             |) where event HE; end
          | HES := HSP ^+ TICK
          | SCAN{}
          | (| HW := ^NEW_BOUND
             | setTextArea(textArea when HW,NEW_BOUND)
             |)/ HW
          |)/ TICK, HS, C_UP_DOWN, C_LIN_EXP, PI_0, textArea, HSP, NEW_BOUND |)
      where 
      constant integer CST_COEF_INIT_PACE = 4;
      constant integer CST_Frame_V1 = 900, CST_Frame_V2= 530;
      constant integer CST_TextArea_V1=100, CST_TextArea_V2=30;
      process SCAN =
           ( ? event HE;
             ! integer PI_0;
               event H_0;
               TWindow textArea;
               TWindow frame;
               event TICK;
               event HS;
               boolean C_LIN_EXP;
               boolean C_UP_DOWN;
           )
         (| (| PERIOD_INPUT{}
             | (| HINIT := H_0
                | MODE_INPUT{}
                |) where event HINIT; end
             | (| h := STOP
                | quitAppli{}
                |) where event h; end
             | INIT_PACE{}
             |)/ butDiv, butMult, butMoins, butPlus, butQuit, butTick, butAuto, STOP |)
         where 
         process MODE_INPUT =
              ( ? TWindow butTick;
                  event HE;
                  TWindow butAuto;
                  event HINIT;
                  TWindow butQuit;
                ! event TICK;
                  event HS;
                  event STOP;
              )
            (| (| AUTOMOD ^= HE
                | AUTOMOD := readCheckBox(butAuto)
                | HS := HE when AUTOMOD
                |)/ AUTOMOD
             | (| C_STOP ^= HE
                | C_STOP := readButton(butQuit)
                | STOP := when C_STOP
                |)/ C_STOP
             | (| C_TICK ^= HE
                | C_TICK := readButton(butTick)
                | TICK := when C_TICK
                |)/ C_TICK
             |)
            where 
            boolean AUTOMOD, C_STOP, C_TICK;
            end
         ; 
         process INIT_PACE =
              ( ? event HE;
                ! integer PI_0;
                  event H_0;
                  TWindow frame;
                  TWindow textArea;
                  TWindow butQuit;
                  TWindow butTick;
                  TWindow butAuto;
                  TWindow butPlus;
                  TWindow butMoins;
                  TWindow butMult;
                  TWindow butDiv;
              )
            (| CINIT := NCINIT$ init true
             | NCINIT := not HE
             | H_0 := when CINIT
             | PI_0 := CST_COEF_INIT_PACE when H_0
             | Frame ^= H_0
             | (Frame,TextArea,ButPlus,ButMoins,ButMult,ButDiv,ButTick,ButAuto,ButQuit) := createPaceMakerWin(H_0)
             | frame := Frame cell HE
             | textArea := TextArea cell HE
             | butQuit := ButQuit cell HE
             | butTick := ButTick cell HE
             | butAuto := ButAuto cell HE
             | butPlus := ButPlus cell HE
             | butMoins := ButMoins cell HE
             | butMult := ButMult cell HE
             | butDiv := ButDiv cell HE
             |)
            where 
            boolean CINIT, NCINIT;
            TWindow Frame, TextArea, ButQuit, ButTick, ButAuto, ButPlus, ButMoins, ButMult, ButDiv;
            process createPaceMakerWin =
                 ( ? event H_0;
                   ! TWindow frame;
                     TWindow textArea;
                     TWindow butPlus;
                     TWindow butMoins;
                     TWindow butMult;
                     TWindow butDiv;
                     TWindow butTick;
                     TWindow butAuto;
                     TWindow butQuit;
                 )
               (| (| H_0 ^= frame
                      | frame := createFrame(CST_Frame_V1,CST_Frame_V2)
                      | p1 := createPanelWithCustomLayout(frame, PaceMakerWinDirection)
                      |)
                   | (| textArea := createTextArea(p1,CST_TextArea_V1,CST_TextArea_V2)
                      | butPlus := createButton(p1)
                      | butMoins := createButton(p1)
                      | butDiv := createButton(p1)
                      | butMult := createButton(p1)
                      | textArea --> butPlus
                      | butPlus --> butMoins
                      | butMoins --> butDiv
                      | butDiv --> butMult
                      |)
                   | (| butTick := createButton(p1)
                      | butTick --> butQuit
                      |)
                   | (| butAuto := createCheckBox(p1,false)
                      | butQuit := createButton(p1)
                      | butAuto --> butTick
                      |)
                   |)
               where 
                 constant integer PaceMakerWinDirection = 1; %  1  -> virtual %
               TWindow p1;
               end
            ; 
            end
         ; 
         event STOP;
         TWindow butAuto, butTick, butQuit, butPlus, butMoins, butMult, butDiv;
         end
      ; 
      process PACE_MAKER =
           ( ? boolean C_UP_DOWN;
               boolean C_LIN_EXP;
               integer PI_0;
               event HE;
             ! integer NEW_BOUND;
               event HSP;
           )
         (| HSP := DYN_EVERY{0,1,1}(0 when HE,NEW_BOUND,HE)
          | SET_BOUND{}
          |)
         where 
         process SET_BOUND =
              ( ? boolean C_UP_DOWN;
                  boolean C_LIN_EXP;
                  integer PI_0;
                ! integer NEW_BOUND;
              )
            (| (| (| NEW_LIN := NEW_BOUND_2
                   | NEW_EXP := NEW_BOUND_1
                   | (| NEW_BOUND ^= C_LIN_EXP ^+ PI_0
                      | NEW_BOUND := PI_0 default NEW_LIN default NEW_EXP default OLD_BOUND
                      |)
                   |) where integer NEW_EXP, NEW_LIN; end
                | OLD_BOUND := NEW_BOUND$ init 1
                | (| EXPONENTIAL{}(C_UP_DOWN when (not C_LIN_EXP),OLD_BOUND when (not C_LIN_EXP))
                   | NEW_BOUND_1 := NEW_BOUND
                   |) where integer NEW_BOUND; end
                | (| LINEAR{}(C_UP_DOWN when C_LIN_EXP,OLD_BOUND when C_LIN_EXP)
                   | NEW_BOUND_2 := NEW_BOUND
                   |) where integer NEW_BOUND; end
                |)/ NEW_BOUND_2, NEW_BOUND_1, OLD_BOUND |)
            where 
            process EXPONENTIAL =
                 ( ? boolean C_UP_DOWN;
                     integer OLD_BOUND;
                   ! integer NEW_BOUND;
                 )
               (| VD := (OLD_BOUND+OLD_BOUND) when (not C_UP_DOWN)
                | VU := (OLD_BOUND/2) when (OLD_BOUND>1) when C_UP_DOWN
                | NEW_BOUND := VU default (VD when (VD>0))
                |)
               where 
               integer VD, VU;
               end
            ; 
            process LINEAR =
                 ( ? boolean C_UP_DOWN;
                     integer OLD_BOUND;
                   ! integer NEW_BOUND;
                 )
               (| DELTA := ((OLD_BOUND/16) when (OLD_BOUND>=16)) default (1 when (^OLD_BOUND))
                | VD := (OLD_BOUND+DELTA) when (not C_UP_DOWN)
                | VU := (OLD_BOUND-DELTA) when (OLD_BOUND>1) when C_UP_DOWN
                | NEW_BOUND := VU default (VD when (VD>0))
                |)
               where 
               integer DELTA, VD, VU;
               end
            ; 
            integer OLD_BOUND, NEW_BOUND_1, NEW_BOUND_2;
            end
         ; 
         process DYN_EVERY =
              { integer PHY_0, PI_0, V_0; }
              ( ? integer PHY;
                  integer PERIOD;
                  event HE;
                ! event HS;
              )
            (| MAX1 := ((0 when (PERIOD<=1)) default (PERIOD-1)) cell HE init (PI_0-1)
             | MAX := MAX1 when HE
             | CPHY1 := ((0 when (PHY<0)) default PHY) cell HE init PHY_0
             | CPHY := CPHY1 when HE
             | ZV := V$ init V_0
             | V := (0 when (ZV>=MAX)) default (ZV+1)
             | HS := when (V=CPHY)
             |)
            where 
            integer MAX1, MAX, CPHY1, CPHY, ZV, V;
            end
         ; 
         end
      ; 
      process PERIOD_INPUT =
           ( ? event HE;
               TWindow butPlus;
               TWindow butMoins;
               TWindow butMult;
               TWindow butDiv;
             ! boolean C_UP_DOWN;
               boolean C_LIN_EXP;
           )
         (| bPlus := readButton(butPlus when HE)
          | bMoins := readButton(butMoins when HE)
          | bMult := readButton(butMult when HE)
          | bDiv := readButton(butDiv when HE)
          | C_LIN_EXP := (true when (bPlus or bMoins)) default (false when (bMult or bDiv))
          | C_UP_DOWN := (true when (bPlus or bMult)) default (false when (bMoins or bDiv))
          |)
         where 
         boolean bPlus, bMoins, bMult, bDiv;
         end
      ; 
      integer NEW_BOUND, PI_0;
      event HSP, HS, TICK, HW;
      TWindow textArea;
      boolean C_LIN_EXP, C_UP_DOWN;
      end
   ; 
end %libGraphics%; 
