package SignalLibraries.Java.graphical;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;

import javax.swing.JPanel;

/**
 * Canvas used for Signal programs simulation, mainly to display curves
 */
public class SimuCanvas extends JPanel {
	
	private static final long	serialVersionUID	= 4369372835125351297L;
	
	private Graphics			gc;
	
	/**
	 * Constructor
	 */
	public SimuCanvas(int x, int y) {
		init(x, y, Color.white);
	}
	
	public SimuCanvas(int x, int y, Color bgColor, Font vfont) {
		init(x, y, bgColor);
		setFont(vfont);
	}
	
	/**
	 * Initialization
	 */
	private void init(int x, int y, Color bgColor) {
		setSize(x, y);
		setVisible(true);
		setBackground(bgColor);
		
		gc = getGraphics();
	}
	
	/**
	 * Draws a line segment
	 * 
	 * @param x1
	 *        x coordinate of the first point
	 * @param y1
	 *        y coordinate of the first point
	 * @param x2
	 *        x coordinate of the second point
	 * @param y2
	 *        y coordinate of the second point
	 */
	public void drawLine(int x1, int y1, int x2, int y2, Color vcolor) {
		gc = getGraphics();
		gc.setColor(vcolor);
		gc.drawLine(x1, y1, x2, y2);
	}
	
	/**
	 * draws a black rectangle
	 * 
	 * @param x
	 *        x coordinate of the origin of the rectangle
	 * @param y
	 *        y coordinate of the origin of the rectangle
	 * @param dx
	 *        rectangle width
	 * @param dy
	 *        rectangle height
	 */
	public void fillRect(int x, int y, int dx, int dy, Color vcolor) {
		gc = getGraphics();
		gc.setColor(vcolor);
		if (dy == 0)
			dy = 1;
		gc.fillRect(x, y, dx, dy);
	}
	
	/**
	 * clears a zone of the rectangle the y coordinate of the origin point must be 0.
	 * 
	 * @param x1
	 *        x coordinate of the origin point
	 * @param dx
	 *        width of the rectangle to clear
	 * @param dy
	 *        height of the rectangle to clear
	 */
	public void clearRect(int x, int y, int dx, int dy) {
		gc = getGraphics();
		gc.setColor(getBackground());
		gc.fillRect(x, y, dx, dy);
	}
	
	/**
	 * displays an integer as a string
	 * 
	 * @param value
	 *        the value to display
	 * @param x
	 *        x coordinate of the point where the string will be displayed in the canvas
	 * @param y
	 *        y coordinate of the point where the string will be displayed in the canvas
	 */
	public void drawInteger(int value, int x, int y, Color vcolor) {
		String chaine = new String("" + value);
		gc = getGraphics();
		gc.setColor(vcolor);
		gc.drawString(chaine, x, y);
	}
	
	public void drawReal(double value, int x, int y, Color vcolor) {
		int ind;
		String chaine = new String("" + value);
		ind = chaine.lastIndexOf(".");
		if (chaine.length() >= ind + 3)
			chaine = chaine.substring(0, ind + 3);
		else
			chaine = chaine.substring(0, ind + 2);
		gc = getGraphics();
		gc.setColor(vcolor);
		gc.drawString(chaine, x, y);
	}
	
}
