module COSTBIB=
% --------
   type __TC__ = enum (_event, _logical, _short, _integer, _long, _real, _dpreal, _complex, _dcomplex, _char,_string);
   type HomoType = integer;
   constant HomoType  NilHomoTypeValue =0 ;
  --------%
  process FILTER =
          ( ? HomoType i;
              event H;
            ! HomoType r;
          )
      (| r := i when H |)
   %FILTER%; 
  process MAX =
          ( ? HomoType IN1;
              HomoType IN2;
            ! HomoType MAX_VAL;
          )
      (| HCOMMON := (when (^IN1)) when (^IN2) 
       | c1 := IN1 when HCOMMON | c2 := IN2 when HCOMMON
       | MAX_VAL := (IN1 when ( c1 > c2)) default IN2 default IN1
       |)
      where 
          event HCOMMON;
          HomoType c1, c2;
      end
   %MAX%; 

   process MEMO = ( ? HomoType i, j; ! HomoType ri; ) 
        (| ri := (i default j) cell ^ri init NilHomoTypeValue |)
        ;

   process MAXCOST = % maxcost of 2 inputs % 
        ( ? HomoType IN1, IN2;
            event H;
          ! HomoType VMAX;
        )
      (| c1 := IN1 when (^IN1) when (^IN2) when H 
       | c2 := IN2 when (^IN1) when (^IN2) when H 
       | VMAX := (IN1 when ( c1 > c2)) default (IN2 when H) default (IN1 when H)
       |)
       where 
          HomoType c1, c2;
       end; 

   process pack = ( ? HomoType i, j; ! HomoType r; )
        (| r := MAXCOST(i,j, ^r) default NilHomoTypeValue |)
        ;

   process waiting = ( ? HomoType i, j; ! HomoType r; ) 
        (| r := MAXCOST(i,j, ^r) default NilHomoTypeValue |)
        ;

   process MAXCOST3 = % maxcost of 3 inputs % 
        ( ? HomoType IN1, IN2, IN3;
            event H;
          ! HomoType VMAX;
        )
        (| VMAX:=MAXCOST(IN1, MAXCOST(IN2, IN3,H) , H) |) 
        ;
        
   process MAXCOST4 = % maxcost of 4 inputs % 
        ( ? HomoType IN1, IN2, IN3, IN4;
            event H;
          ! HomoType VMAX;
        )
        (| VMAX:=MAXCOST( IN1, MAXCOST3(IN2,IN3, IN4, H), H) |)
        ;

   
   process CostRenaming =  
           % X:= Y % 
           { __TC__ VTYPE; } 
           ( ? HomoType IN1, CLKCOST, WAIT; 
           ! HomoType OUT, DONE; )
       (| OUT := MAXCOST3(IN1, CLKCOST, WAIT, ^OUT)
        | DONE := MEMO(OUT, WAIT)
        |);


% --SYNCHRONOUS  OPERATORS % 

   process CostConst =
        { __TC__ VTYPE; }
        ( ! HomoType OUT;
        )
      (| OUT := NilHomoTypeValue %GET_COST_CONST(VTYPE)% |)
   ;

   %---- Unary % 
   process SYNCHRONOUS_UNARYOP =
        ( ? HomoType IN1, CLKCOST, WAIT;
            HomoType COST_OP;
          ! HomoType OUT, DONE;
        )
      (| X := IN1 when (IN1 > CLKCOST) default CLKCOST 
       | VMAXU := MAX( X, WAIT when ^OUT) 
       | OUT := VMAXU +  COST_OP
       | DONE := MEMO(OUT, WAIT)
       |)
      where 
      HomoType VMAXU , X ;
      end
      ;

   process CostNot = 
           ( ? HomoType IN1, CLKCOST, WAIT;
             ! HomoType OUT, DONE; 
           )
        (| (OUT, DONE) := SYNCHRONOUS_UNARYOP(IN1, CLKCOST, WAIT, GET_COST_NOT()) |)
   %CostNot%; 

   process CostUMinus =
        { __TC__ VTYPE; }
        ( ? HomoType IN1, CLKCOST, WAIT;
          ! HomoType OUT, DONE;
        )
        (| (OUT, DONE) := SYNCHRONOUS_UNARYOP(IN1,CLKCOST, WAIT, GET_COST_UMINUS(VTYPE)) |)
   %CostUMinus%; 

   %---- Binary % 

   process SYNCHRONOUS_BINOP =
        ( ? HomoType IN1;
            HomoType IN2;
            HomoType CLKCOST;
            HomoType WAIT;
            HomoType COST_OP;
          ! HomoType OUT, DONE;
        )
      (| X := IN1 when ((IN1 > IN2) and (IN1>CLKCOST))  default IN2 when (IN2 > CLKCOST) default CLKCOST
       | VMAXB := MAX( X, WAIT when ^OUT)
       | OUT := VMAXB + COST_OP
       | DONE := MEMO(OUT, WAIT)
       |)
      where 
      HomoType VMAXB, X ;
      end ;
   process CostEQ =
        { __TC__ VTYPE1; __TC__ VTYPE2;}
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
            HomoType WAIT; 
          ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_RELA(VTYPE1,VTYPE2) %GET_COST_EQ(VTYPE1, VTYPE2) % ) |)
   %CostEQ%; 
   process CostNEQ =
        { __TC__ VTYPE1; __TC__ VTYPE2;}
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_NEQ(VTYPE1, VTYPE2) ) |)
   %CostNEQ%
   ; 
   process CostLEQ =
        { __TC__ VTYPE1; __TC__ VTYPE2;}
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_LEQ(VTYPE1, VTYPE2)) |)
   %CostLEQ%; 

   process CostGEQ =
        { __TC__ VTYPE1; __TC__ VTYPE2;}
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_GEQ(VTYPE1, VTYPE2)) |)
   %CostGEQ%; 

   process CostLT =
        { __TC__ VTYPE1; __TC__ VTYPE2;}
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_LT(VTYPE1, VTYPE2)) |)
   %CostLT%; 

   process CostGT =
        { __TC__ VTYPE1; __TC__ VTYPE2;}
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_GT(VTYPE1, VTYPE2)) |)
   %CostGT%; 

   process CostPlus =
        { __TC__ VTYPE1; __TC__ VTYPE2;}
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
            HomoType WAIT; 
          ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_PLUS{VTYPE1, VTYPE2} () ) |)
   %CostPlus%; 

   process CostMinus =
        { __TC__ VTYPE1; __TC__ VTYPE2;}
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_MINUS(VTYPE1, VTYPE2)) |)
   %CostMinus%; 

   process CostMult =
        { __TC__ VTYPE1; __TC__ VTYPE2;}
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_MULT(VTYPE1, VTYPE2)) |)
   %CostMult%; 

   process CostDivi =
        { __TC__ VTYPE1; __TC__ VTYPE2;}
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_DIVI(VTYPE1, VTYPE2)) |)
   %CostDivi%; 

   process CostPow =
        { __TC__ VTYPE1;} % defined only for integers?, value is needed x**6 %
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_POW(VTYPE1)) |)
   %CostDivi%; 


   process CostAnd =
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
      (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT,   CST_COST_AND %GET_COST_AND() % ) |)
   %CostAnd%; 

   process CostOr =
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
     (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1, IN2, CLKCOST, WAIT, CST_COST_OR %GET_COST_AND() % ) |)
   %CostOr%; 

   process CostEor =
        ( ? HomoType IN1;
            HomoType IN2, CLKCOST;
           HomoType WAIT; ! HomoType OUT, DONE;
        )
     (| (OUT, DONE) := SYNCHRONOUS_BINOP(IN1,IN2, CLKCOST, WAIT, GET_COST_EOR()) |)
   %CostOr%; 


% --POLYCHRONOUS OPERATORS % 
   process CostWhen1 =
        ( ? HomoType IN1; 
            HomoType CLKCOST, WAIT;
        ! HomoType OUT, DONE;
        )
      (| VMAXW := MAXCOST3(IN1, CLKCOST, WAIT, ^OUT)
       | OUT := VMAXW + CST_COST_TEST %GET_COST_WHEN1() %
       | DONE := MEMO(OUT, WAIT)
       |)
      where
      HomoType VMAXW; 
      end;

   process CostWhen =
        { __TC__ VTYPE; }
        ( ? HomoType IN1;
            HomoType CLKCOST, WAIT;
          ! HomoType OUT, DONE;
        )
      (| VMAXW := MAXCOST3(IN1, CLKCOST, WAIT, ^OUT)
       | OUT := VMAXW +  CST_COST_TEST %GET_COST_WHEN(VTYPE) %
       | DONE := MEMO(OUT, WAIT)
       |)
      where 
      HomoType VMAXW;
      end
   %CostWhen%; 

   process CostDefault =
        { __TC__ VTYPE; }
        ( ? HomoType IN1;
            HomoType IN2;
            HomoType CLKCOST, WAIT;
          ! HomoType OUT, DONE;
        )
        pragmas Comment " OUT := max( ^CLKCOST, IN1 default IN2, WAIT) + Delta(default) " end pragmas

        (| X := IN1 default IN2 
         | Y := X when (X > CLKCOST) default CLKCOST
         | VMAXD := MAXCOST(Y, WAIT,^OUT)
         | OUT := VMAXD + GET_COST_DEFAULT(VTYPE)
         | DONE := MEMO(OUT, WAIT)
%         | VMAXD := MAXCOST4(IN1, IN2, CLKCOST, WAIT, ^OUT) % 
       |)
      where 
        HomoType VMAXD, X, Y;
      end
   %CostDefault%; 

   process CostEvent =
        { __TC__ VTYPE; }
        ( ? HomoType IN1;
          ! HomoType OUT;
        )
      (| OUT := IN1+GET_COST_EVENT() |)
   %CostEvent%; 

   process CostCell =
        { __TC__ VTYPE; }
        ( ? HomoType IN1; 
            HomoType IN2;
            HomoType CLKCOST, WAIT;
          ! HomoType OUT, DONE;
        )
     (  % | (OUT, DONE) := CostDefault{VTYPE} (IN1, OUTDoll, CLKCOST, WAIT) | OUTDoll := ( OUT $1 init NilHomoTypeValue) |) %
   
       | VMAXC:= MAXCOST4(IN1, IN2 , CLKCOST, WAIT, ^OUT)
       | OUT := VMAXC + GET_COST_CELL(VTYPE)
       | DONE := MEMO(OUT, WAIT)                      
       |)
      where 
      HomoType   VMAXC %OUTDoll % ;
      end
   %CostCell%; 

   process CostLeftTT =
           ( ? HomoType IN1;
               HomoType IN2;
               HomoType CLKCOST, WAIT;
             ! HomoType OUT, DONE;
            )
           (| VMAXL := MAXCOST4(IN1,IN2, CLKCOST, WAIT, ^OUT)
            | OUT := VMAXL + GET_COST_LEFT_TT()
            | DONE := MEMO(OUT, WAIT)
            |)
      where 
      HomoType VMAXL;
      end ;

   process CostDelay =
        { __TC__ VTYPE; }
        ( ? HomoType IN1, CLKCOST, WAIT; 
          ! HomoType OUT, DONE; )
         (| VMAXDelay := MAXCOST3((IN1 $1 init NilHomoTypeValue) + GET_COST_DELAY_WR(VTYPE), CLKCOST, WAIT, ^OUT )
          | OUT := VMAXDelay   + GET_COST_DELAY_RD(VTYPE) 
          | DONE := MEMO(OUT, WAIT)
          |)
      where 
      HomoType VMAXDelay ;
      end ;

   process CostCount =
        { __TC__ VTYPE; }
        ( ? HomoType IN1, IN2, CLKCOST, WAIT; 
          ! HomoType OUT, DONE; )
        (| X := IN1 when (IN1>CLKCOST)  default CLKCOST
         | VMAXCount := MAX( X, WAIT when ^OUT)
         | OUT := VMAXCount  + GET_COST_PLUS{VTYPE, VTYPE}() + GET_COST_MODULO()
         | DONE := MEMO(OUT, WAIT)
         |)
      where 
      HomoType VMAXCount, X;
      end ;

   process CostSubGraph=
        ( ? HomoType IN1;
            HomoType IN2;
            HomoType CLKCOST, WAIT;
          ! HomoType OUT, DONE;
        )
        (| OUT := MAXCOST4(IN1, IN2, CLKCOST, WAIT, ^OUT)
         | DONE := MEMO(OUT, WAIT)
         |)
        ;

   process CostInput =
         { __TC__ VTYPE; }	
         ( ? HomoType IN1, CLKCOST, WAIT; 
           ! HomoType OUT, DONE; )
        (| VMAXI := MAXCOST(IN1, WAIT, ^CLKCOST)
         | OUT := VMAXI + GET_COST_READ(VTYPE)
         | DONE := MEMO(OUT, WAIT)
         |)
      where 
        HomoType VMAXI;
      end ;

   process CostOutput =
         { __TC__ VTYPE; }	
         ( ? HomoType IN1, CLKCOST, WAIT; 
           ! HomoType OUT, DONE; )
        (| VMAXO := MAXCOST(IN1, WAIT, ^CLKCOST)
         | OUT := VMAXO + GET_COST_WRITE(VTYPE)
         | DONE := MEMO(OUT, WAIT)
         |)
      where 
       HomoType VMAXO;
      end ;

   process CostFieldAccess =
         ( ? HomoType IN1, CLKCOST, WAIT; 
           ! HomoType OUT, DONE; )
        (| VMAXF := MAXCOST(IN1, WAIT, ^CLKCOST)
         | OUT := VMAXF  % + GET_COST_WRITE(VTYPE) % 
         | DONE := MEMO(OUT, WAIT)
         |)
        where 
             HomoType VMAXF;
        end 
      ;

      % LA SUITE EST A REVOIR % 
      % --------------------- % 

   process CostComm =
        { __TC__ VTYPE; }
        ( ? HomoType IN_;
          ! HomoType OUT;
        )
      (| OUT := IN_+GET_COST_COMM(VTYPE) |)
   %CostComm%; 
   process DEPENDENCE =
        ( ? HomoType IN1;
            HomoType IN2;
            event DepClk;
          ! HomoType OUT;
        )
      (| OUT := MAXCOST(IN1,IN2,DepClk) |)
      ;
end;
