module fifoLib =

process fifo_reset =
{type T; integer size; T initVal;}
( ? T input;
    event reset;
  ! T output;
    integer counter; )
 pragmas
   Comment 
   "It models a FIFO, bounded to size, of elements of type T, initialized at initVal."
   "On reset, the FIFO is reinitialized."
   "An element (input) is added when it is present and the FIFO is not full."
   "An element (output) is returned on the clock of output when the FIFO is not empty,"
   "otherwise the previous output is returned."
 end pragmas
(| counter := CounterOf{size} (input, output, reset)
 | syncCounter := counter cell tick
 | buffer := (input window (size+1)) cell tick
 | output := ((zdefaultVal when (syncCounter=0))
         default (buffer[size-syncCounter] when ^input)
         default (buffer[size+1-syncCounter])) when ^output
 | defaultVal := output
             default (initVal when reset)
             default (buffer[0] when ^input when (syncCounter=size))
             default zdefaultVal
 | zdefaultVal := defaultVal $1 init initVal
 | defaultVal ^= tick
 | tick :=  input ^+ reset ^+ output ^+ counter
 |)
where
  event readAndWrite, writeOnFull, readOnEmpty, tick;
  [size+1] T buffer;
  T defaultVal, zdefaultVal;
  integer nextCounter, syncCounter;
end;
  
  
process fifo =
{type T; integer size; T initVal;}
(? T input;
 ! T output;
   integer counter; )
 pragmas
   Comment 
   "A fifo is a fifo_reset with null clock as reset"
 end pragmas
  (| (output, counter) := fifo_reset{T,size, initVal}(input, ^0) |)
;
  
end;
